@@ -5,10 +5,11 @@ package SReview::Config;
 
 use Data::Dumper;
 use Carp;
+use Mojo::JSON qw/decode_json encode_json/;
 
 =head1 NAME
 
-SReview::Config - Self-reproducing and self-documentating configuration file system
+SReview::Config - Self-reproducing and self-documenting configuration file system
 
 =head1 SYNOPSIS
 
@@ -43,7 +44,9 @@ sub new {
 	my $cfile = shift;
 
 	if (! -f $cfile) {
-		carp "Warning: could not find configuration file $cfile, falling back to defaults";
+		unless (grep /^SREVIEW_/, keys(%ENV)) {
+			carp "Warning: could not find configuration file $cfile, falling back to defaults";
+		}
 	} else {
 		package SReview::Config::_private;
 		use Carp;
@@ -80,8 +83,57 @@ sub define {
 	}
 	$self->{defs}{$name}{doc} = $doc;
 	$self->{defs}{$name}{default} = $default;
+	my $NAME = uc $name;
+	if(exists($ENV{"SREVIEW_${NAME}"})) {
+		$self->set($name => decode_json($ENV{"SREVIEW_${NAME}"}));
+	}
 };
 
+=head2 $config->define_deprecated(oldname, newname, conversion_sub)
+
+Define a name as a deprecated way of configuring things. When this value
+is set, SReview::Config will issue a warning that this option is now
+deprecated, and that the user should use some other option instead.
+
+The conversion subroutine is an optional argument that should mangle the
+value given to "oldname" into the value expected by "newname". If it
+returns nonzero, then SReview::Config will croak. It will receive a
+reference to the "config" object, the value that is trying to be set, and the
+name of the new parameter.
+
+The default conversion sub just sets the value of the newname
+configuration without any conversion.
+
+=cut
+
+sub define_deprecated {
+	my $self = shift;
+	my $oldname = shift;
+	my $newname = shift;
+	my $convert = shift;
+
+	if(exists($self->{fixed})) {
+		croak "Tried to define a new value after a value has already been requested. This is not allowed!";
+	}
+	$self->{defs}{$oldname}{deprecated} = 1;
+	$self->{defs}{$oldname}{instead} = $newname;
+	if(defined($convert)) {
+		$self->{defs}{$oldname}{convert} = $convert;
+	} else {
+		$self->{defs}{$oldname}{convert} = sub { my $self = shift; my $old = shift; $self->set($newname => $old); return 0; };
+	}
+	if(exists($SReview::Config::_private::{$oldname})) {
+		carp "Found a value for \"$oldname\" when it was being defined as a deprecated name. Please convert this value to a value of $newname!\n";
+		if ((&$self->{defs}{$oldname}{convert}($self, $SReview::Config::_private::{$oldname}, $newname)) != 0) {
+			croak "Could not convert deprecated value to new name: $!";
+		}
+	}
+	my $NAME = uc $oldname;
+	if(exists($ENV{"SREVIEW_${NAME}"})) {
+		$self->set($newname => decode_json($ENV{"SREVIEW_${NAME}"}));
+	}
+}
+
 sub define_computed {
 	my $self = shift;
 	my $name = shift;
@@ -97,7 +149,9 @@ definitions of this configuration file; that is, once this method has
 been called, the C<define> method above will croak.
 
 The returned value will either be the default value configured at
-C<define> time, or the value configured in the configuration file.
+C<define> time, the value configured in the configuration file, or the
+value set in the environment variable C<SREVIEW_I<name> >, where I<name>
+is the upper-case version of the name of the configuration item.
 
 =cut
 
@@ -116,12 +170,11 @@ sub get {
 	$self->{fixed} = 1;
 	if(exists($SReview::Config::_private::{$name})) {
 		return ${$SReview::Config::_private::{$name}};
-	} else {
-		if(defined($ENV{'SREVIEW_VERBOSE'}) && $ENV{'SREVIEW_VERBOSE'} gt 0) {
-			print "No configuration value found for $name, using defaults\n";
-		}
-		return $self->{defs}{$name}{default};
 	}
+	if(defined($ENV{'SREVIEW_VERBOSE'}) && $ENV{'SREVIEW_VERBOSE'} gt 0) {
+		print "No configuration value found for $name, using defaults\n";
+	}
+	return $self->{defs}{$name}{default};
 };
 
 =head2 $config->set('name', value);
@@ -142,6 +195,14 @@ sub set {
 		}
 		{
 			my $val = $vals{$name};
+			if(exists($self->{defs}{$name}{deprecated})) {
+				my $newname = $self->{defs}{$name}{instead};
+				carp "A value for \"$name\" is being set, which is a deprecated name for $newname. Please update things for the new value\n";
+				if ((&{$self->{defs}{$name}{convert}}($self, $vals{$name}, $self->{defs}{$name}{instead})) != 0) {
+					croak "Could not convert deprecated value for $name to $newname format\n";
+				}
+				return;
+			}
 			$SReview::Config::_private::{$name} = \$val;
 		}
 	}
@@ -208,6 +269,32 @@ sub dump {
 
 =back
 
+=head2 $config->dump_item("item")
+
+Print a JSON value for the given configuration item. Prints the default
+item if this item hasn't been set a value.
+
+=cut
+
+sub dump_item {
+	my ($self, $item) = @_;
+
+	print encode_json($self->get($item));
+}
+
+=head2 $config->is_default("item")
+
+Return a truthy value if the given configuration item is still at its
+default value.
+
+=cut
+
+sub is_default {
+	my ($self, $item) = @_;
+
+	return (exists($SReview::Config::_private::{$item})) ? 0 : 1;
+}
+
 =head1 BUGS
 
 It is currently not possible to load more than one configuration file in
