@@ -650,6 +650,24 @@ sub by_nonce {
 	return $rv;
 }
 
+=head2 by_slug
+
+Looks up (and returns) the talk by slug, rather than by talk ID
+
+=cut
+
+sub by_slug {
+	my $klass = shift;
+	my $slug = shift;
+
+	my $st = $pg->db->dbh->prepare("SELECT * FROM talks WHERE slug = ?");
+	$st->execute($slug);
+	die "Talk does not exist (or the slug is not unique in the database).\n" unless $st->rows == 1;
+	my $row = $st->fetchrow_arrayref;
+	my $rv = SReview::Talk->new(talkid => $row->[0], slug => $slug);
+	return $rv;
+}
+
 =head2 add_correction
 
 Interpret a correction as a number, and add the passed parameter to it.
